package cn.com.qmth.examcloud.exchange.outer.api.provider;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.questions.api.ExtractConfigCloudService;
import cn.com.qmth.examcloud.core.questions.api.request.SaveRandomPaperExtractConfigReq;
import cn.com.qmth.examcloud.exchange.outer.api.ExtractConfigOuterService;
import cn.com.qmth.examcloud.exchange.outer.api.request.OuterSaveExtractConfigReq;
import cn.com.qmth.examcloud.exchange.outer.api.response.OuterSaveExtractConfigResp;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

@ApiIgnore
@Api(tags = "调卷规则相关接口")
@RestController
@RequestMapping("${$rmp.cloud.exchange.outer}/extract_config")
public class ExtractConfigOuterServiceProvider extends ControllerSupport implements ExtractConfigOuterService {

    private static final Logger log = LoggerFactory.getLogger(ExtractConfigOuterServiceProvider.class);

    private static final long serialVersionUID = 1949134276130329162L;

    @Autowired
    private ExtractConfigCloudService extractConfigCloudService;

    @ApiOperation(value = "保存调卷规则（千卷考试）", hidden = true)
    @PostMapping("random/paper/save")
    @Override
    public OuterSaveExtractConfigResp saveRandomPaperExtractConfig(
            @RequestBody @ApiParam(required = true) OuterSaveExtractConfigReq req) {
        if (StringUtils.isBlank(req.getExamCode()) && StringUtils.isBlank(req.getExamName())) {
            throw new StatusException("examCode、examCode不能都为空");
        }

        Long rootOrgId = getEnterpriseRootOrgId();
        log.warn("==>随机抽卷调卷规则 creating... rootOrgId:{}  examCode:{} examName:{} courseCode:{}",
                rootOrgId, req.getExamCode(), req.getExamName(), req.getCourseCode());

        SaveRandomPaperExtractConfigReq savereq = new SaveRandomPaperExtractConfigReq();
        savereq.setRootOrgId(rootOrgId);
        savereq.setExamName(req.getExamName());
        savereq.setExamCode(req.getExamCode());
        savereq.setCourseCode(req.getCourseCode());
        extractConfigCloudService.saveRandomPaperExtractConfig(savereq);

        OuterSaveExtractConfigResp resp = new OuterSaveExtractConfigResp();
        return resp;
    }

}
